@@ -1,12 +1,13 @@
 #!/usr/bin/env python3
 # This file is part of Xpra.
-# Copyright (C) 2010-2024 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2010 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import sys
 import os.path
 from time import monotonic
+from collections.abc import Sequence
 from typing import Any, NoReturn
 
 from xpra.os_util import gi_import
@@ -14,7 +15,7 @@ from xpra.util.system import SIGNAMES
 from xpra.util.str_fn import csv
 from xpra.util.env import envint, envbool, envfloat
 from xpra.audio.audio_pipeline import AudioPipeline
-from xpra.gtk.gobject import n_arg_signal
+from xpra.util.gobject import n_arg_signal
 from xpra.gstreamer.common import normv, has_plugins, plugin_str, GST_FLOW_OK, \
     get_default_appsink_attributes, get_element_str
 from xpra.audio.gstreamer_util import (
@@ -52,7 +53,8 @@ class AudioSource(AudioPipeline):
         "new-buffer": n_arg_signal(3),
     }
 
-    def __init__(self, src_type="", src_options=None, codecs=(), codec_options=None, volume=1.0):
+    def __init__(self, src_type: str, src_options: dict[str, int | float | str],
+                 codecs: Sequence[str], codec_options: dict, volume=1.0):
         if not src_type:
             try:
                 from xpra.audio.pulseaudio.util import get_pa_device_options
@@ -99,8 +101,6 @@ class AudioSource(AudioPipeline):
         self.jitter_queue = None
         self.container_format = (fmt or "").replace("mux", "").replace("pay", "")
         self.stream_compressor = stream_compressor
-        if src_options is None:
-            src_options = {}
         src_options["name"] = "src"
         source_str = plugin_str(src_type, src_options)
         # FIXME: this is ugly and relies on the fact that we don't pass any codec options to work!
@@ -369,10 +369,11 @@ def main() -> int:
             output = sys.stdout
         else:
             output = open(filename, "wb")
-        ss = AudioSource(codecs=[codec])
+
+        ss = AudioSource("", src_options={}, codecs=[codec], codec_options={})
         lock = Lock()
 
-        def new_buffer(_audiosource, data, metadata, packet_metadata):
+        def new_buffer(_audiosource, data, metadata, packet_metadata) -> None:
             log.info("new buffer: %s bytes (%s), metadata=%s", len(data), type(data), metadata)
             with lock:
                 for x in packet_metadata:
