@@ -1,5 +1,5 @@
 # This file is part of Xpra.
-# Copyright (C) 2019-2024 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2019 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -30,7 +30,7 @@ from xpra.log import Logger
 log = Logger("gobject", "client")
 
 REFRESH_RATE = envint("XPRA_REFRESH_RATE", 1)
-CURSES_LOG = os.environ.get("XPRA_CURSES_LOG")
+CURSES_LOG = os.environ.get("XPRA_CURSES_LOG", "")
 
 WHITE = 0
 GREEN = 1
@@ -41,8 +41,9 @@ EXIT_KEYS = (ord("q"), ord("Q"))
 PAUSE_KEYS = (ord("p"), ord("P"))
 SIGNAL_KEYS = {
     3: signal.SIGINT,
-    26: signal.SIGSTOP,
 }
+if hasattr(signal, "SIGSTOP"):
+    SIGNAL_KEYS[26] = signal.SIGSTOP
 
 
 def get_title() -> str:
@@ -57,13 +58,18 @@ def curses_init():
     curses.noecho()
     curses.raw()
     curses.start_color()
-    curses.use_default_colors()
+    if curses.can_change_color():
+        try:
+            curses.use_default_colors()
+            curses.init_pair(WHITE, curses.COLOR_WHITE, curses.COLOR_BLACK)
+            curses.init_pair(GREEN, curses.COLOR_GREEN, curses.COLOR_BLACK)
+            curses.init_pair(YELLOW, curses.COLOR_YELLOW, curses.COLOR_BLACK)
+            curses.init_pair(RED, curses.COLOR_RED, curses.COLOR_BLACK)
+        except Exception:
+            # this can fail on some terminals, ie: mingw
+            pass
     # for i in range(0, curses.COLORS):
     #    curses.init_pair(i+1, i, -1)
-    curses.init_pair(WHITE, curses.COLOR_WHITE, curses.COLOR_BLACK)
-    curses.init_pair(GREEN, curses.COLOR_GREEN, curses.COLOR_BLACK)
-    curses.init_pair(YELLOW, curses.COLOR_YELLOW, curses.COLOR_BLACK)
-    curses.init_pair(RED, curses.COLOR_RED, curses.COLOR_BLACK)
     return stdscr
 
 
@@ -159,6 +165,22 @@ def get_window_info(wi: typedict) -> Sequence[tuple[str, int]]:
     return tuple((x, WHITE) for x in info if x)
 
 
+def get_proxy_info_str(proxy_info: dict) -> str:
+    if not proxy_info:
+        return ""
+    td = typedict(proxy_info)
+    proxy_platform_info = typedict(td.dictget("platform", {}))
+    proxy_platform = proxy_platform_info.strget("")
+    proxy_release = proxy_platform_info.strget("release")
+    proxy_build_info = typedict(td.dictget("build", {}))
+    proxy_version = proxy_build_info.strget("version")
+    proxy_distro = td.strget("linux_distribution")
+    return " via: %s proxy version %s" % (
+        platform_name(proxy_platform, proxy_distro or proxy_release),
+        std(proxy_version or "unknown")
+    )
+
+
 class TopClient:
 
     def __init__(self, opts):
@@ -168,7 +190,7 @@ class TopClient:
         self.position = 0
         self.selected_session = None
         self.message = None
-        self.exit_code = None
+        self.exit_code : ExitValue | None = None
         self.dotxpra = DotXpra(self.socket_dir, self.socket_dirs)
         self.last_getch = 0
         self.psprocess = {}
@@ -181,14 +203,14 @@ class TopClient:
         self.cleanup()
         return self.exit_code or 0
 
-    def signal_handler(self, signum, *_args):
+    def signal_handler(self, signum: int, *_args) -> None:
         self.exit_code = 128 + signum
 
-    def setup(self):
+    def setup(self) -> None:
         self.stdscr = curses_init()
         curses.cbreak()
 
-    def cleanup(self):
+    def cleanup(self) -> None:
         scr = self.stdscr
         if scr:
             curses.nocbreak()
@@ -196,11 +218,13 @@ class TopClient:
             curses_clean(scr)
             self.stdscr = None
 
-    def update_loop(self):
+    def update_loop(self) -> None:
         while self.exit_code is None:
             self.update_screen()
+            # elapsed is in milliseconds:
             elapsed = int(1000 * monotonic() - self.last_getch)
-            delay = max(100, min(1000, 1000 - elapsed)) // 100
+            # delay is in tenths of a second (1 to 10 here):
+            delay = max(1, min(1000, 1000 - elapsed) // 100)
             curses.halfdelay(delay)
             try:
                 v = self.stdscr.getch()
@@ -224,7 +248,7 @@ class TopClient:
             elif v in (ord("d"), ord("D")):
                 self.run_subcommand("detach")
 
-    def show_selected_session(self):
+    def show_selected_session(self) -> None:
         # show this session:
         try:
             self.cleanup()
@@ -251,23 +275,22 @@ class TopClient:
         finally:
             self.setup()
 
-    def run_subcommand(self, subcommand):
+    def run_subcommand(self, subcommand: str) -> Popen | None:
         return self.do_run_subcommand(subcommand, stdout=DEVNULL, stderr=DEVNULL)
 
-    def do_run_subcommand(self, subcommand, **kwargs):
+    def do_run_subcommand(self, subcommand: str, **kwargs) -> Popen | None:
         cmd = get_nodock_command() + [subcommand, self.selected_session]
         try:
             return Popen(cmd, **kwargs)
         except Exception:
             return None
 
-    def update_screen(self) -> bool:
+    def update_screen(self) -> None:
         self.stdscr.erase()
         try:
             self.do_update_screen()
         finally:
             self.stdscr.refresh()
-        return True
 
     def do_update_screen(self) -> None:
         # c = self.stdscr.getch()
@@ -398,6 +421,7 @@ class TopSessionClient(InfoTimerClient):
 
     def __init__(self, *args):
         super().__init__(*args)
+        self.client_type = "top"
         self.log_file = None
         if CURSES_LOG:
             self.log_file = open(CURSES_LOG, "ab")  # pylint: disable=consider-using-with
@@ -407,9 +431,6 @@ class TopSessionClient(InfoTimerClient):
         self.psprocess = {}
         start_thread(self.input_thread, "input-thread", daemon=True)
 
-    def client_type(self) -> str:
-        return "top"
-
     def server_connection_established(self, caps) -> bool:
         self.log(f"server_connection_established({caps!r})")
         self.log("traceback: " + str(traceback.extract_stack()))
@@ -417,6 +438,14 @@ class TopSessionClient(InfoTimerClient):
         self.update_screen()
         return super().server_connection_established(caps)
 
+    def print_desktop_size(self, c: typedict) -> None:
+        # skip this as it would interfere with curses
+        pass
+
+    def print_server_info(self, c: typedict) -> None:
+        # skip this as it would interfere with curses
+        pass
+
     def setup(self) -> None:
         if self.stdscr is None:
             self.stdscr = curses_init()
@@ -513,7 +542,6 @@ class TopSessionClient(InfoTimerClient):
         # if c==curses.KEY_RESIZE:
         height, width = self.stdscr.getmaxyx()
         title = get_title()
-        sli = self.server_last_info
 
         def _addstr(pad: int, py: int, px: int, s: str, *args) -> None:
             if len(s) + px >= width - pad:
@@ -531,139 +559,34 @@ class TopSessionClient(InfoTimerClient):
             addstr_main(0, x, title, curses.A_BOLD)
             if height <= 1:
                 return
-            server_info = self.slidictget("server")
-            build = self.slidictget("server", "build")
-            vstr = caps_to_version(build)
-            mode = server_info.strget("mode", "server")
-            python_info = self.td(server_info.dictget("python", {}))
-            bits = python_info.intget("bits", 0)
-            bitsstr = "" if bits == 0 else f" {bits}-bit"
-            server_str = f"Xpra {mode} server version {vstr}{bitsstr}"
-            proxy_info = self.slidictget("proxy")
-            if proxy_info:
-                proxy_platform_info = self.td(proxy_info.dictget("platform", {}))
-                proxy_platform = proxy_platform_info.strget("")
-                proxy_release = proxy_platform_info.strget("release")
-                proxy_build_info = self.td(proxy_info.dictget("build", {}))
-                proxy_version = proxy_build_info.strget("version")
-                proxy_distro = proxy_info.strget("linux_distribution")
-                server_str += " via: %s proxy version %s" % (
-                    platform_name(proxy_platform, proxy_distro or proxy_release),
-                    std(proxy_version or "unknown")
-                )
-            addstr_main(1, 0, server_str)
+            addstr_main(1, 0, self.get_server_str())
             if height <= 2:
                 return
-            # load and uptime:
-            now = datetime.now()
-            uptime = ""
-            elapsed_time = server_info.intget("elapsed_time")
-            if elapsed_time:
-                td = timedelta(seconds=elapsed_time)
-                uptime = " up " + str(td).lstrip("0:")
-            clients_info = self.slidictget("clients")
-            nclients = clients_info.intget("")
-            load_average = ""
-            load = sli.inttupleget("load")
-            if load and len(load) == 3:
-                float_load = tuple(v / 1000.0 for v in load)
-                load_average = ", load average: %1.2f, %1.2f, %1.2f" % float_load
-            addstr_main(2, 0, "xpra top - %s%s, %2i users%s" % (
-                now.strftime("%H:%M:%S"), uptime, nclients, load_average))
+            addstr_main(2, 0, self.get_summary_str())
             if height <= 3:
                 return
-            thread_info = self.slidictget("threads")
-            thread_count = thread_info.intget("count")
-            rinfo = f"{thread_count} threads"
-            server_pid = server_info.intget("pid", 0)
-            if server_pid:
-                rinfo += f", pid {server_pid}"
-                machine_id = server_info.get("machine-id")
-                if machine_id is None or machine_id == get_machine_id():
-                    try:
-                        process = self.psprocess.get(server_pid)
-                        if not process:
-                            import psutil
-                            process = psutil.Process(server_pid)
-                            self.psprocess[server_pid] = process
-                        else:
-                            cpu = process.cpu_percent()
-                            rinfo += f", {cpu:3}% CPU"
-                    except Exception:
-                        pass
-            cpuinfo = self.slidictget("cpuinfo")
-            if cpuinfo:
-                rinfo += ", " + cpuinfo.strget("hz_actual")
             elapsed = monotonic() - self.server_last_info_time
-            color = WHITE
-            if self.server_last_info_time == 0:
-                rinfo += " - no server data"
-            elif elapsed > 2:
-                rinfo += f" - last updated {elapsed} seconds ago"
-                color = RED
-            addstr_main(3, 0, rinfo, curses.color_pair(color))
+            color = WHITE if (self.server_last_info_time or elapsed <= 2) else RED
+            addstr_main(3, 0, self.get_machine_str(), curses.color_pair(color))
             if height <= 4:
                 return
-            # display:
-            dinfo = []
-            server = self.slidictget("server")
-            rws = server.intpair("root_window_size", None)
-            display_info = self.slidictget("display")
-            if rws:
-                rww, rwh = rws
-                sinfo = f"{rww}x{rwh}"
-                depth = display_info.intget("depth")
-                if depth > 0:
-                    sinfo += f" {depth}-bit"
-                sinfo += " display"
-                mds = server.intpair("max_desktop_size")
-                if mds:
-                    mdw, mdh = mds
-                    sinfo += f" (max {mdw}x{mdh})"
-                dinfo.append(sinfo)
-            cursor_info = self.slidictget("cursor")
-            if cursor_info:
-                cx, cy = cursor_info.inttupleget("position", (0, 0))
-                dinfo.append(f"cursor at {cx}x{cy}")
-            pid = display_info.intget("pid")
-            if pid:
-                dinfo.append(f"pid {pid}")
-            addstr_main(4, 0, csv(dinfo))
+            addstr_main(4, 0, self.get_display_str())
             if height <= 5:
                 return
             hpos = 5
-            gl_info = self.get_gl_info(display_info.dictget("opengl"))
+            gl_info = self.get_gl_info(self.slidictget("display").dictget("opengl"))
             if gl_info:
                 addstr_main(5, 0, gl_info)
                 hpos += 1
 
             # filter clients, only show GUI clients:
             client_info = self.slidictget("client")
-            gui_clients = []
-            nclients = 0
-            while True:
-                ci = client_info.dictget(nclients)
-                if not ci:
-                    break
-                ci = self.td(ci)
-                session_id = ci.strget("session-id")
-                if session_id != self.session_id and ci.boolget("windows", True) and ci.strget("type") != "top":
-                    gui_clients.append(nclients)
-                nclients += 1
+            nclients, gui_clients = self.get_gui_clients()
 
             ngui = 0
             if hpos < height - 3:
                 hpos += 1
-                if nclients == 0:
-                    clients_str = "no clients connected"
-                else:
-                    ngui = len(gui_clients)
-                    clients_str = f"{nclients} clients connected, "
-                    if ngui == 0:
-                        clients_str += "no gui clients"
-                    else:
-                        clients_str += f"{ngui} gui clients:"
-                addstr_main(hpos, 0, clients_str)
+                addstr_main(hpos, 0, self.get_clients_str(nclients, gui_clients))
                 hpos += 1
             for client_index, client_no in enumerate(gui_clients):
                 ci = client_info.dictget(client_no)
@@ -708,6 +631,119 @@ class TopSessionClient(InfoTimerClient):
         except Exception as e:
             self.err(e)
 
+    def get_server_str(self) -> str:
+        server_info = self.slidictget("server")
+        build = self.slidictget("server", "build")
+        vstr = caps_to_version(build)
+        mode = server_info.strget("mode", "server")
+        python_info = self.td(server_info.dictget("python", {}))
+        bits = python_info.intget("bits", 0)
+        bitsstr = "" if bits == 0 else f" {bits}-bit"
+        server_str = f"Xpra {mode} server version {vstr}{bitsstr}"
+        server_str += get_proxy_info_str(self.slidictget("proxy"))
+        return server_str
+
+    def get_summary_str(self) -> str:
+        # load and uptime:
+        now = datetime.now()
+        uptime = ""
+        server_info = self.slidictget("server")
+        elapsed_time = server_info.intget("elapsed_time")
+        if elapsed_time:
+            td = timedelta(seconds=elapsed_time)
+            uptime = " up " + str(td).lstrip("0:")
+        clients_info = self.slidictget("clients")
+        nclients = clients_info.intget("")
+        load_average = ""
+        load = self.server_last_info.inttupleget("load")
+        if load and len(load) == 3:
+            float_load = tuple(v / 1000.0 for v in load)
+            load_average = ", load average: %1.2f, %1.2f, %1.2f" % float_load
+        return "xpra top - %s%s, %2i users%s" % (now.strftime("%H:%M:%S"), uptime, nclients, load_average)
+
+    def get_machine_str(self) -> str:
+        thread_info = self.slidictget("threads")
+        thread_count = thread_info.intget("count")
+        rinfo = f"{thread_count} threads"
+        server_info = self.slidictget("server")
+        server_pid = server_info.intget("pid", 0)
+        if server_pid:
+            rinfo += f", pid {server_pid}"
+            machine_id = server_info.get("machine-id")
+            if machine_id is None or machine_id == get_machine_id():
+                try:
+                    process = self.psprocess.get(server_pid)
+                    if not process:
+                        import psutil
+                        process = psutil.Process(server_pid)
+                        self.psprocess[server_pid] = process
+                    else:
+                        cpu = process.cpu_percent()
+                        rinfo += f", {cpu:3}% CPU"
+                except Exception:
+                    pass
+        cpuinfo = self.slidictget("cpuinfo")
+        if cpuinfo:
+            rinfo += ", " + cpuinfo.strget("hz_actual")
+        elapsed = monotonic() - self.server_last_info_time
+        if self.server_last_info_time == 0:
+            rinfo += " - no server data"
+        elif elapsed > 2:
+            rinfo += f" - last updated {elapsed} seconds ago"
+        return rinfo
+
+    def get_display_str(self) -> str:
+        # display:
+        dinfo = []
+        server = self.slidictget("server")
+        rws = server.intpair("root_window_size", None)
+        display_info = self.slidictget("display")
+        if rws:
+            rww, rwh = rws
+            sinfo = f"{rww}x{rwh}"
+            depth = display_info.intget("depth")
+            if depth > 0:
+                sinfo += f" {depth}-bit"
+            sinfo += " display"
+            mds = server.intpair("max_desktop_size")
+            if mds:
+                mdw, mdh = mds
+                sinfo += f" (max {mdw}x{mdh})"
+            dinfo.append(sinfo)
+        cursor_info = self.slidictget("cursor")
+        if cursor_info:
+            cx, cy = cursor_info.inttupleget("position", (0, 0))
+            dinfo.append(f"cursor at {cx}x{cy}")
+        pid = display_info.intget("pid")
+        if pid:
+            dinfo.append(f"pid {pid}")
+        return csv(dinfo)
+
+    def get_gui_clients(self) -> tuple[int, Sequence]:
+        client_info = self.slidictget("client")
+        gui_clients = []
+        nclients = 0
+        while True:
+            if nclients not in client_info:
+                break
+            ci = self.td(client_info.dictget(nclients))
+            session_id = ci.strget("session-id")
+            if session_id != self.session_id and ci.boolget("windows", True) and ci.strget("type") != "top":
+                gui_clients.append(nclients)
+            nclients += 1
+        return nclients, gui_clients
+
+    def get_clients_str(self, nclients: int, gui_clients: Sequence) -> str:
+        if nclients == 0:
+            return "no clients connected"
+        ngui = len(gui_clients)
+        clients_str = f"{nclients} clients connected, "
+        if ngui == 0:
+            clients_str += "no gui clients"
+        else:
+            clients_str += f"{ngui} gui clients:"
+        return clients_str
+
     def slidictget(self, *parts) -> typedict:
         return self.dictget(self.server_last_info, *parts)
 
@@ -808,7 +844,7 @@ class TopSessionClient(InfoTimerClient):
             return "av-sync: disabled by client"
         return "av-sync: enabled - video delay: %ims" % (avsi.intget("total", 0))
 
-    def get_gl_info(self, gli) -> str:
+    def get_gl_info(self, gli: dict) -> str:
         if not gli:
             return ""
         gli = self.td(gli)
