@@ -1,7 +1,7 @@
 # This file is part of Xpra.
 # Copyright (C) 2011 Serviware (Arthur Huillet, <ahuillet@serviware.com>)
-# Copyright (C) 2010-2024 Antoine Martin <antoine@xpra.org>
-# Copyright (C) 2008, 2010 Nathaniel Smith <njs@pobox.com>
+# Copyright (C) 2010 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2008 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -12,10 +12,10 @@ from collections.abc import Callable, MutableSequence
 
 from xpra.os_util import OSX, WIN32, gi_import
 from xpra.client.gui.widget_base import ClientWidgetBase
-from xpra.client.gui.window_backing_base import fire_paint_callbacks
+from xpra.client.gui.window.backing import fire_paint_callbacks
 from xpra.client.gui.window_border import WindowBorder
 from xpra.net.common import PacketElement
-from xpra.common import gravity_str, WORKSPACE_UNSET, WORKSPACE_NAMES, force_size_constraint
+from xpra.common import gravity_str, force_size_constraint
 from xpra.util.parsing import scaleup_value, scaledown_value
 from xpra.util.system import is_Wayland
 from xpra.util.objects import typedict
@@ -26,9 +26,7 @@ from xpra.log import Logger
 GLib = gi_import("GLib")
 
 log = Logger("window")
-plog = Logger("paint")
-focuslog = Logger("focus")
-mouselog = Logger("mouse")
+paintlog = Logger("paint")
 workspacelog = Logger("workspace")
 keylog = Logger("keyboard")
 metalog = Logger("metadata")
@@ -37,7 +35,6 @@ iconlog = Logger("icon")
 alphalog = Logger("alpha")
 scalinglog = Logger("scaling")
 
-SIMULATE_MOUSE_DOWN = envbool("XPRA_SIMULATE_MOUSE_DOWN", True)
 PROPERTIES_DEBUG = [x.strip() for x in os.environ.get("XPRA_WINDOW_PROPERTIES_DEBUG", "").split(",")]
 SET_SIZE_CONSTRAINTS = envbool("XPRA_SET_SIZE_CONSTRAINTS", True)
 DEFAULT_GRAVITY = envint("XPRA_DEFAULT_GRAVITY", 0)
@@ -51,20 +48,34 @@ def is_wm_property(name: str) -> bool:
     return any(name.startswith(prefix) for prefix in ("_MOTIF", "WM_", "_NET_WM", "_GTK_"))
 
 
+def validhostname(value):
+    if value in (
+            "localhost",
+            "localhost.localdomain",
+            "",
+            None,
+    ):
+        return None
+    return value
+
+
 class ClientWindowBase(ClientWidgetBase):
 
-    def __init__(self, client, group_leader, watcher_pid: int, wid: int,
-                 wx: int, wy: int, ww: int, wh: int, bw: int, bh: int,
-                 metadata: typedict, override_redirect: bool, client_properties,
-                 border: WindowBorder, max_window_size, default_cursor_data, pixel_depth: int,
+    def __init__(self, client, group_leader, wid: int,
+                 geom: tuple[int, int, int, int],
+                 backing_size: tuple[int, int],
+                 metadata: typedict, override_redirect: bool, client_properties: typedict,
+                 border: WindowBorder, max_window_size, pixel_depth: int,
                  headerbar="no"):
         log("%s%s", type(self),
-            (client, group_leader, watcher_pid, wid,
-             wx, wy, ww, wh, bw, bh,
+            (client, group_leader, wid,
+             geom, backing_size,
              metadata, override_redirect, client_properties,
-             border, max_window_size, default_cursor_data, pixel_depth,
+             border, max_window_size, pixel_depth,
              headerbar))
-        super().__init__(client, watcher_pid, wid, metadata.boolget("has-alpha"))
+        super().__init__(client, wid, metadata.boolget("has-alpha"))
+        wx, wy, ww, wh = geom
+        bw, bh = backing_size
         self._override_redirect = override_redirect
         self.group_leader = group_leader
         self._pos = (wx, wy)
@@ -88,12 +99,10 @@ class ClientWindowBase(ClientWidgetBase):
         self._iconified = False
         self._focused = False
         self._opaque_region = ()
+        self._actions = ()
         self.window_gravity = OVERRIDE_GRAVITY or DEFAULT_GRAVITY
         self.border = border
-        self.cursor_data = None
-        self.default_cursor_data = default_cursor_data
         self.max_window_size = max_window_size
-        self.button_state: dict[int, bool] = {}
         self.pixel_depth = pixel_depth  # 0 for default
         # window_offset is the delta between the location of the window requested by the server,
         # and where we actually end up mapping it on the client
@@ -102,35 +111,22 @@ class ClientWindowBase(ClientWidgetBase):
         self.pending_refresh: list[tuple[int, int, int, int]] = []
         self.headerbar = headerbar
 
-        self.init_window(metadata)
+        self.init_window(client, metadata, client_properties)
         self.setup_window(bw, bh)
         self.update_metadata(metadata)
         self.finalize_window()
 
     def __repr__(self):
-        return f"ClientWindow({self.wid})"
+        return f"ClientWindow({self.wid:#x})"
 
-    def init_window(self, metadata: typedict) -> None:
+    def init_window(self, client, metadata: typedict, client_props: typedict) -> None:
         self._backing = None
         self._metadata = typedict()
         # used for only sending focus events *after* the window is mapped:
         self._been_mapped = False
 
-        def wn(w):
-            return WORKSPACE_NAMES.get(w, w)
-
-        workspace = typedict(self._client_properties).intget("workspace", -1)
-        workspacelog("init_window(..) workspace from client properties %s: %s", self._client_properties, wn(workspace))
-        if workspace >= 0:
-            # client properties override application specified workspace value on init only:
-            metadata["workspace"] = workspace
-        self._window_workspace = WORKSPACE_UNSET  # will get set in set_metadata if present
-        self._desktop_workspace = self.get_desktop_workspace()  # pylint: disable=assignment-from-none
-        workspacelog("init_window(..) workspace=%s, current workspace=%s",
-                     wn(self._window_workspace), wn(self._desktop_workspace))
         if self.max_window_size and "size-constraints" not in metadata:
             # this ensures that we will set size-constraints and honour max_window_size:
-            metadata.pop("workspace", None)
             metadata["size-constraints"] = {}
         # initialize gravity early:
         sc = typedict(metadata.dictget("size-constraints", {}))
@@ -157,7 +153,7 @@ class ClientWindowBase(ClientWidgetBase):
             # "group-leader"          : self.group_leader,
             "position": self._pos,
             "size": self._size,
-            "client-properties": self._client_properties,
+            "client-properties": dict(self._client_properties),
             "set-initial-position": self._set_initial_position,
             "size-constraints": dict(self.size_constraints),
             "geometry-hints": dict(self.geometry_hints),
@@ -165,19 +161,11 @@ class ClientWindowBase(ClientWidgetBase):
             "attributes": attributes,
             "gravity": gravity_str(self.window_gravity),
             # "border"                : self.border or "",
-            # cursor_data
             "max-size": self.max_window_size,
-            "button-state": self.button_state,
             "offset": self.window_offset or (0, 0),
         }
         return info
 
-    def get_desktop_workspace(self):
-        return None
-
-    def get_window_workspace(self):
-        return None
-
     ######################################################################
     # screen scaling:
     def fsx(self, v) -> float:
@@ -227,7 +215,8 @@ class ClientWindowBase(ClientWidgetBase):
         w, h = self._size
         self._backing = self.make_new_backing(backing_class, w, h, bw, bh)
         self._backing.border = self.border
-        self._backing.default_cursor_data = self.default_cursor_data
+        # soft dependency on `PointerWindow`:
+        self._backing.default_cursor_data = getattr(self, "default_cursor_data", ())
         self._backing.gravity = self.window_gravity
         # this is only used by cairo to request a repaint for the fps counter:
         self._backing.repaint = self.repaint
@@ -276,14 +265,8 @@ class ClientWindowBase(ClientWidgetBase):
     def apply_transient_for(self, wid: int) -> None:
         raise NotImplementedError
 
-    def paint_spinner(self, context, area) -> None:
-        raise NotImplementedError
-
-    def _pointer_modifiers(self, event):
-        raise NotImplementedError
-
-    def xget_u32_property(self, target, name):
-        raise NotImplementedError
+    def xget_u32_property(self, target, name: str, default_value=0) -> int:
+        return default_value
 
     def is_OR(self) -> bool:
         return self._override_redirect
@@ -295,97 +278,89 @@ class ClientWindowBase(ClientWidgetBase):
         self._metadata.update(metadata)
         try:
             self.set_metadata(metadata)
-        except Exception:
+        except RuntimeError:
             metalog.warn("failed to set window metadata to '%s'", metadata, exc_info=True)
 
-    def _get_window_title(self, metadata) -> str:
+    def _get_window_title(self, metadata: typedict) -> str:
         try:
-            title = self._client.title.replace("\0", "")
-            if title.find("@") < 0:
-                return title
-            # perform metadata variable substitutions:
-            # full of Python 3 unicode headaches that don't need to be
-            UNKNOWN_MACHINE = "<unknown machine>"
-            remote_hostname = getattr(self._client, "_remote_hostname", None)
-            remote_display = getattr(self._client, "_remote_display", None)
-            if remote_display:
-                # ie: "1\\WinSta0\\Default" -> 1-WinSta0-Default
-                remote_display = remote_display.replace("\\", "-")
-            default_values = {
-                "title": "<untitled window>",
-                "client-machine": UNKNOWN_MACHINE,
-                "windowid": str(self.wid),
-                "server-machine": std(remote_hostname) or UNKNOWN_MACHINE,
-                "server-display": std(remote_display) or "<unknown display>",
-            }
-            metalog(f"default values: {default_values}")
-
-            def validhostname(value):
-                if value in (
-                        "localhost",
-                        "localhost.localdomain",
-                        "",
-                        None,
-                ):
-                    return None
-                return value
-
-            def getvar(var):
-                # "hostname" is magic:
-                # we try harder to find a useful value to show:
-                if var in ("hostname", "hostinfo"):
-                    server_display = getattr(self._client, "server_display", None)
-                    if var == "hostinfo" and getattr(self._client, "mmap_enabled", False) and server_display:
-                        # this is a local connection for sure and we can specify the display directly:
-                        return server_display
-                    import socket
-                    local_hostname = validhostname(socket.gethostname())
-                    # try to find the hostname:
-                    proto = getattr(self._client, "_protocol", None)
-                    target = None
-                    if proto:
-                        conn = getattr(proto, "_conn", None)
-                        if conn:
-                            hostname = conn.info.get("host")
-                            target = str(conn.target)
-                            if hostname:
-                                if local_hostname == hostname and server_display:
-                                    return server_display
-                                return hostname
-                    for m in ("client-machine", "server-machine"):
-                        value = validhostname(getvar(m))
-                        if value and value != local_hostname:
-                            return value
-                    if server_display:
-                        return server_display
-                    if target:
-                        return target
-                    return UNKNOWN_MACHINE
-                value = metadata.get(var) or self._metadata.get(var)
-                if value is None:
-                    return default_values.get(var, "<unknown %s>" % var)
-                return str(value)
-
-            def metadata_replace(match):
-                atvar = match.group(0)  # ie: '@title@'
-                var = atvar[1:len(atvar) - 1]  # ie: 'title'
-                if not var:
-                    # atvar = "@@"
-                    return "@"
-                return getvar(var)
-
-            sub = r"@[\w\-]*@"
-            replaced = re.sub(sub, metadata_replace, title)
-            metalog("re.sub%s=%s", (sub, metadata_replace, title), replaced)
-            return replaced
+            return self.do_get_window_title(metadata)
         except Exception as e:
             log.error("Error parsing window title:")
             log.estr(e)
-            return ""
+            return self._client.title.replace("\0", "")
+
+    def do_get_window_title(self, metadata: typedict) -> str:
+        title = self._client.title.replace("\0", "")
+        if title.find("@") < 0:
+            return title
+        # perform metadata variable substitutions:
+        UNKNOWN_MACHINE = "<unknown machine>"
+        remote_hostname = getattr(self._client, "_remote_hostname", None)
+        remote_display = getattr(self._client, "_remote_display", None)
+        if remote_display:
+            # ie: "1\\WinSta0\\Default" -> 1-WinSta0-Default
+            remote_display = remote_display.replace("\\", "-")
+        default_values: dict[str, str] = {
+            "title": "<untitled window>",
+            "client-machine": UNKNOWN_MACHINE,
+            "windowid": str(self.wid),
+            "server-machine": std(remote_hostname) or UNKNOWN_MACHINE,
+            "server-display": std(remote_display) or "<unknown display>",
+        }
+        metalog(f"default values: {default_values}")
+
+        def getvar(var) -> str:
+            # "hostname" is magic:
+            # we try harder to find a useful value to show:
+            if var in ("hostname", "hostinfo"):
+                server_display = getattr(self._client, "server_display", None)
+                if var == "hostinfo" and getattr(self._client, "mmap_enabled", False) and server_display:
+                    # this is a local connection for sure and we can specify the display directly:
+                    return server_display
+                import socket
+                local_hostname = validhostname(socket.gethostname())
+                # try to find the hostname:
+                proto = getattr(self._client, "_protocol", None)
+                target = None
+                if proto:
+                    conn = getattr(proto, "_conn", None)
+                    if conn:
+                        hostname = conn.info.get("host")
+                        target = str(conn.target)
+                        if hostname:
+                            if local_hostname == hostname and server_display:
+                                return server_display
+                            return hostname
+                for m in ("client-machine", "server-machine"):
+                    value = validhostname(getvar(m))
+                    if value and value != local_hostname:
+                        return value
+                if server_display:
+                    return server_display
+                if target:
+                    return target
+                return UNKNOWN_MACHINE
+            value = metadata.get(var) or self._metadata.get(var)
+            if value is None:
+                return default_values.get(var, "<unknown %s>" % var)
+            return str(value)
+
+        def metadata_replace(match) -> str:
+            atvar = match.group(0)  # ie: '@title@'
+            var = atvar[1:len(atvar) - 1]  # ie: 'title'
+            if not var:
+                # atvar = "@@"
+                return "@"
+            return getvar(var)
+
+        sub = r"@[\w\-]*@"
+        replaced = re.sub(sub, metadata_replace, title)
+        metalog("re.sub%s=%s", (sub, metadata_replace, title), replaced)
+        return replaced
 
     def set_metadata(self, metadata: typedict):
         metalog("set_metadata(%s)", metadata)
-        debug_props = [x for x in PROPERTIES_DEBUG if x in metadata.keys()]
+        debug_props = tuple(x for x in PROPERTIES_DEBUG if x in metadata.keys())
         for x in debug_props:
             metalog.info("set_metadata: %s=%s", x, metadata.get(x))
         # WARNING: "class-instance" needs to go first because others may realize the window
@@ -398,6 +373,10 @@ class ClientWindowBase(ClientWidgetBase):
             title = self._get_window_title(metadata)
             self.set_title(title)
 
+        if "locale" in metadata:
+            locale = metadata.get("locale", "")
+            self.set_locale(locale)
+
         if "icon-title" in metadata:
             icon_title = metadata.strget("icon-title")
             self.set_icon_name(icon_title)
@@ -416,32 +395,24 @@ class ClientWindowBase(ClientWidgetBase):
             self._set_initial_position = metadata.boolget("set-initial-position")
 
         if "transient-for" in metadata:
-            wid = metadata.intget("transient-for", -1)
-            self.apply_transient_for(wid)
+            self.apply_transient_for(metadata.intget("transient-for"))
 
         if "modal" in metadata:
-            modal = metadata.boolget("modal")
-            self.set_modal(modal)
+            self.set_modal(metadata.boolget("modal"))
 
         # apply window-type hint if window has not been mapped yet:
-        if "window-type" in metadata and not self.get_mapped():
-            window_types = metadata.strtupleget("window-type")
-            self.set_window_type(window_types)
+        if "window-type" in metadata:
+            self.set_window_type(metadata.strtupleget("window-type"))
 
         if "role" in metadata:
-            role = metadata.strget("role")
-            self.set_role(role)
+            self.set_role(metadata.strget("role"))
 
         if "xid" in metadata:
-            xid = metadata.strget("xid")
-            self.set_xid(xid)
+            self.set_xid(metadata.strget("xid"))
 
         if "opacity" in metadata:
             opacity = metadata.intget("opacity", -1)
-            if opacity < 0:
-                opacity = 1
-            else:
-                opacity = min(1, opacity // 0xffffffff)
+            opacity = 1 if opacity < 0 else min(1, opacity // 0xffffffff)
             ignorewarnings(self.set_opacity, opacity)
 
         if "has-alpha" in metadata:
@@ -455,28 +426,13 @@ class ClientWindowBase(ClientWidgetBase):
                 self._has_alpha = new_alpha
 
         if "maximized" in metadata:
-            maximized = metadata.boolget("maximized")
-            if maximized != self._maximized:
-                self._maximized = maximized
-                if maximized:
-                    self.maximize()
-                else:
-                    self.unmaximize()
+            self.set_maximized(metadata.boolget("maximized"))
 
         if "fullscreen" in metadata:
-            fullscreen = metadata.boolget("fullscreen")
-            if self._fullscreen is None or self._fullscreen != fullscreen:
-                self._fullscreen = fullscreen
-                self.set_fullscreen(fullscreen)
+            self.set_fullscreen(metadata.boolget("fullscreen"))
 
         if "iconic" in metadata:
-            iconified = metadata.boolget("iconic")
-            if self._iconified != iconified:
-                self._iconified = iconified
-                if iconified:
-                    self.iconify()
-                else:
-                    self.deiconify()
+            self.set_iconic(metadata.boolget("iconic"))
 
         if "decorations" in metadata:
             decorated = metadata.boolget("decorations", True)
@@ -489,49 +445,28 @@ class ClientWindowBase(ClientWidgetBase):
                 self.apply_geometry_hints(self.geometry_hints)
 
         if "above" in metadata:
-            above = metadata.boolget("above")
-            if self._above != above:
-                self._above = above
-                self.set_keep_above(above)
+            self.set_above(metadata.boolget("above"))
 
         if "below" in metadata:
-            below = metadata.boolget("below")
-            if self._below != below:
-                self._below = below
-                self.set_keep_below(below)
+            self.set_below(metadata.boolget("below"))
 
         if "shaded" in metadata:
-            shaded = metadata.boolget("shaded")
-            if self._shaded != shaded:
-                self._shaded = shaded
-                self.set_shaded(shaded)
+            self.set_shaded(metadata.boolget("shaded"))
 
         if "sticky" in metadata:
-            sticky = metadata.boolget("sticky")
-            if self._sticky != sticky:
-                self._sticky = sticky
-                if sticky:
-                    self.stick()
-                else:
-                    self.unstick()
+            self.set_sticky(metadata.boolget("sticky"))
 
         if "skip-taskbar" in metadata:
-            skip_taskbar = metadata.boolget("skip-taskbar")
-            if self._skip_taskbar != skip_taskbar:
-                self._skip_taskbar = skip_taskbar
-                self.set_skip_taskbar_hint(skip_taskbar)
+            self.set_skip_taskbar(metadata.boolget("skip-taskbar"))
 
         if "skip-pager" in metadata:
-            skip_pager = metadata.boolget("skip-pager")
-            if self._skip_pager != skip_pager:
-                self._skip_pager = skip_pager
-                self.set_skip_pager_hint(skip_pager)
+            self.set_skip_pager(metadata.boolget("skip-pager"))
+
+        if "focused" in metadata:
+            self.set_focused(metadata.boolget("focused"))
 
         if "opaque-region" in metadata:
-            opaque_region = metadata.tupleget("opaque-region", None, 0)
-            if self._opaque_region != opaque_region:
-                self._opaque_region = opaque_region
-                self.set_opaque_region(opaque_region)
+            self.set_opaque_region(metadata.tupleget("opaque-region"))
 
         if "workspace" in metadata:
             self.set_workspace(metadata.intget("workspace"))
@@ -561,29 +496,86 @@ class ClientWindowBase(ClientWidgetBase):
         if "content-type" in metadata:
             self.content_type = metadata.strget("content-type")
 
+        if "actions" in metadata:
+            self._actions = metadata.strtupleget("actions")
+
     def set_x11_property(self, *x11_property) -> None:
         pass  # see gtk client window base
 
-    def set_command(self, command) -> None:
+    def set_above(self, above: bool) -> None:
+        if self._above != above:
+            self._above = above
+            self.set_keep_above(above)
+
+    def set_below(self, below: bool) -> None:
+        if self._below != below:
+            self._below = below
+            self.set_keep_below(below)
+
+    def set_sticky(self, sticky: bool) -> None:
+        if self._sticky != sticky:
+            self._sticky = sticky
+            if sticky:
+                self.stick()
+            else:
+                self.unstick()
+
+    def set_skip_taskbar(self, skip_taskbar: bool) -> None:
+        if self._skip_taskbar != skip_taskbar:
+            self._skip_taskbar = skip_taskbar
+            self.set_skip_taskbar_hint(skip_taskbar)
+
+    def set_skip_pager(self, skip_pager: bool) -> None:
+        if self._skip_pager != skip_pager:
+            self._skip_pager = skip_pager
+            self.set_skip_pager_hint(skip_pager)
+
+    def set_focused(self, focused: bool) -> None:
+        # only implemented for X11
+        pass
+
+    def set_opaque_region(self, rectangles: tuple | None) -> None:
         pass  # see gtk client window base
 
+    def set_iconic(self, iconified: bool) -> None:
+        if self._iconified != iconified:
+            self._iconified = iconified
+            if iconified:
+                self.iconify()
+            else:
+                self.deiconify()
+
+    def set_fullscreen(self, fullscreen: bool) -> None:
+        """ see gtk3 window for implementation """
+
+    def set_maximized(self, maximized: bool) -> None:
+        if maximized != self._maximized:
+            self._maximized = maximized
+            if maximized:
+                self.maximize()
+            else:
+                self.unmaximize()
+
+    def set_command(self, command) -> None:
+        """ see gtk3 window for implementation """
+
     def set_class_instance(self, wmclass_name, wmclass_class) -> None:
-        pass  # see gtk client window base
+        """ see gtk3 window for implementation """
 
     def set_shape(self, shape) -> None:
-        log("set_shape(%s) not implemented by %s", shape, type(self))
+        """ see gtk3 window for implementation """
 
     def set_bypass_compositor(self, v) -> None:
-        pass  # see gtk client window base
+        """ see gtk3 window for implementation """
 
     def set_strut(self, strut) -> None:
-        pass  # see gtk client window base
+        """ see gtk3 window for implementation """
 
     def set_fullscreen_monitors(self, fsm) -> None:
-        pass  # see gtk client window base
+        """ see gtk3 window for implementation """
 
     def set_shaded(self, shaded) -> None:
-        pass  # see gtk client window base
+        """ see gtk3 window for implementation """
 
     def reset_size_constraints(self) -> None:
         self.set_size_constraints(self.size_constraints, self.max_window_size)
@@ -694,15 +686,15 @@ class ClientWindowBase(ClientWidgetBase):
         if b:
             b.gravity = self.window_gravity
 
+    def set_locale(self, locale: str) -> None:
+        pass  # see gtk client window base
+
     def set_window_type(self, window_types) -> None:
         pass  # see gtk client window base
 
     def set_workspace(self, workspace) -> None:
         pass  # see gtk client window base
 
-    def set_fullscreen(self, fullscreen) -> None:
-        pass  # see gtk client window base
-
     def set_xid(self, xid) -> None:
         pass  # see gtk client window base
 
@@ -811,7 +803,7 @@ class ClientWindowBase(ClientWidgetBase):
         raise NotImplementedError("no repaint on %s" % type(self))
 
     def refresh_window(self, *args) -> None:
-        log("refresh_window(%s) wid=%s", args, self.wid)
+        log("refresh_window(%s) wid=%#x", args, self.wid)
         self._client.send_refresh(self.wid)
 
     def refresh_all_windows(self, *_args) -> None:
@@ -842,7 +834,7 @@ class ClientWindowBase(ClientWidgetBase):
 
     def after_draw_refresh(self, success, message="") -> None:
         backing = self._backing
-        plog(f"after_draw_refresh({success}, {message!r}) pending_refresh={self.pending_refresh}, {backing=}")
+        paintlog(f"after_draw_refresh({success}, {message!r}) pending_refresh={self.pending_refresh}, {backing=}")
         if not backing:
             return
         if backing.repaint_all or self._xscale != 1 or self._yscale != 1 or is_Wayland():
@@ -865,122 +857,29 @@ class ClientWindowBase(ClientWidgetBase):
         if backing:
             backing.eos()
 
-    def spinner(self, _ok) -> None:
-        if not self.can_have_spinner():
+    def set_alert_state(self, alert_state: bool) -> None:
+        can = self.can_have_alert()
+        backing = self._backing
+        log("set_alert_state(%s) can_have_alert=%s, backing=%s", alert_state, can, backing)
+        if not can or not backing:
             return
-        log("spinner(%s) queueing redraw")
-        # with normal windows, we just queue a draw request
-        # and let the expose event paint the spinner
-        w, h = self.get_size()
-        self.repaint(0, 0, w, h)
+        backing.alert_state = alert_state
 
-    def can_have_spinner(self) -> bool:
+    def can_have_alert(self) -> bool:
         if self._backing is None:
             return False
         window_types = self._metadata.strtupleget("window-type")
         return bool(set(window_types) & SHOW_SPINNER_WINDOW_TYPES)
 
-    def _focus(self) -> bool:
-        return self._client.update_focus(self.wid, True)
+    def redraw(self):
+        # queue a repaint request
+        w, h = self.get_size()
+        self.repaint(0, 0, w, h)
 
     def _unfocus(self) -> bool:
+        # overriden in FocusWindow
         return self._client.update_focus(self.wid, False)
 
-    def quit(self) -> None:
-        self._client.quit(0)
-
-    def void(self) -> None:
-        """
-        This method can be used to capture key shortcuts
-        without triggering any specific action.
-        """
-
-    def show_window_info(self, *_args) -> None:
-        from xpra.client.gtk3.window_info import WindowInfo
-        wi = WindowInfo(self._client, self)
-        wi.show()
-
-    def show_session_info(self, *args) -> None:
-        self._client.show_session_info(*args)
-
-    def show_menu(self, *args) -> None:
-        self._client.show_menu(*args)
-
-    def show_start_new_command(self, *args) -> None:
-        self._client.show_start_new_command(*args)
-
-    def show_bug_report(self, *args) -> None:
-        self._client.show_bug_report(*args)
-
-    def show_file_upload(self, *args) -> None:
-        self._client.show_file_upload(*args)
-
-    def show_shortcuts(self, *args) -> None:
-        self._client.show_shortcuts(*args)
-
-    def show_docs(self, *args) -> None:
-        self._client.show_docs(*args)
-
     @staticmethod
     def log(message="") -> None:
         log.info(message)
-
-    def next_keyboard_layout(self, update_platform_layout) -> None:
-        self._client.next_keyboard_layout(update_platform_layout)
-
-    def keyboard_layout_changed(self, *args) -> None:
-        # used by win32 hooks to tell us about keyboard layout changes for this window
-        keylog("keyboard_layout_changed%s", args)
-        self._client.window_keyboard_layout_changed(self)
-
-    def get_mouse_event_wid(self, *_args) -> int:
-        # used to be overridden in GTKClientWindowBase
-        return self.wid
-
-    def _do_motion_notify_event(self, event) -> None:
-        if self._client.readonly or self._client.server_readonly or not self._client.server_pointer:
-            return
-        pointer_data, modifiers, buttons = self._pointer_modifiers(event)
-        wid = self.get_mouse_event_wid(*pointer_data)
-        mouselog("do_motion_notify_event(%s) wid=%s / focus=%s / window wid=%i",
-                 event, wid, self._client._focused, self.wid)
-        mouselog(" device=%s, pointer=%s, modifiers=%s, buttons=%s",
-                 self._device_info(event), pointer_data, modifiers, buttons)
-        device_id = 0
-        self._client.send_mouse_position(device_id, wid, pointer_data, modifiers, buttons)
-
-    def _device_info(self, event) -> str:
-        try:
-            return event.device.get_name()
-        except AttributeError:
-            return ""
-
-    def _button_action(self, button: int, event, depressed: bool, props=None) -> None:
-        if self._client.readonly or self._client.server_readonly or not self._client.server_pointer:
-            return
-        pointer_data, modifiers, buttons = self._pointer_modifiers(event)
-        wid = self.get_mouse_event_wid(*pointer_data)
-        mouselog("_button_action(%s, %s, %s) wid=%s / focus=%s / window wid=%i",
-                 button, event, depressed, wid, self._client._focused, self.wid)
-        mouselog(" device=%s, pointer=%s, modifiers=%s, buttons=%s",
-                 self._device_info(event), pointer_data, modifiers, buttons)
-        device_id = 0
-
-        def send_button(pressed, **kwargs):
-            sprops = props or {}
-            sprops.update(kwargs)
-            self._client.send_button(device_id, wid, button, pressed, pointer_data, modifiers, buttons, sprops)
-
-        pressed_state = self.button_state.get(button, False)
-        if SIMULATE_MOUSE_DOWN and pressed_state is False and depressed is False:
-            mouselog("button action: simulating missing mouse-down event for window %s before mouse-up", wid)
-            # (needed for some dialogs on win32):
-            send_button(True, synthetic=True)
-        self.button_state[button] = depressed
-        send_button(depressed)
-
-    def do_button_press_event(self, event) -> None:
-        self._button_action(event.button, event, True)
-
-    def do_button_release_event(self, event) -> None:
-        self._button_action(event.button, event, False)
