@@ -1,5 +1,5 @@
 # This file is part of Xpra.
-# Copyright (C) 2011-2024 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2011 Antoine Martin <antoine@xpra.org>
 # Copyright (C) 2010 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
@@ -21,7 +21,7 @@ from xpra.util.str_fn import csv, strtobytes, bytestostr, Ellipsizer
 from xpra.util.env import envint
 from xpra.common import noop
 from xpra.util.stats import values_to_scaled_values, values_to_diff_scaled_values, to_std_unit, std_unit_dec, std_unit
-from xpra.client.gui import features
+from xpra.client.base import features
 from xpra.client.base.command import InfoTimerClient
 from xpra.gtk.window import add_close_accel
 from xpra.gtk.graph import make_graph_imagesurface
@@ -61,6 +61,31 @@ def bool_icon(image, on_off: bool):
     image.set_from_pixbuf(icon)
 
 
+def setall(labels: Sequence[Gtk.Label], values: Sequence) -> None:
+    assert len(labels) == len(values), "%s labels and %s values (%s vs %s)" % (
+        len(labels), len(values), labels, values)
+    for i, l in enumerate(labels):
+        l.set_text(str(values[i]))
+
+
+def setlabels(labels: Sequence[Gtk.Label], values: Sequence, rounding: Callable = int):
+    if not values:
+        return
+    avg = sum(values) / len(values)
+    svalues = sorted(values)
+    l = len(svalues)
+    assert l > 0
+    if l < 10:
+        index = l - 1
+    else:
+        index = int(l * 90 / 100)
+    index = max(0, min(l - 1, index))
+    pct = svalues[index]
+    disp = values[-1], min(values), avg, pct, max(values)
+    rounded_values = [rounding(v) for v in disp]
+    setall(labels, rounded_values)
+
+
 def pixelstr(v) -> str:
     if v < 0:
         return "n/a"
@@ -134,7 +159,7 @@ def newdictlook(d, parts, fallback=None):
     return v
 
 
-def image_label_hbox():
+def image_label_hbox() -> tuple:
     hbox = Gtk.HBox(homogeneous=False, spacing=10)
     image_widget = Gtk.Image()
     image_widget.set_size_request(48, 48)
@@ -148,7 +173,7 @@ def image_label_hbox():
     return image_widget, label_widget, hbox
 
 
-def settimedeltastr(label_widget, from_time):
+def settimedeltastr(label_widget, from_time) -> None:
     import time
     delta = datetime.timedelta(seconds=(int(time.time()) - int(from_time)))
     label_widget.set_text(str(delta))
@@ -189,7 +214,7 @@ def get_local_platform_name() -> str:
 
 
 def get_server_builddate(client) -> str:
-    build_info = client.server_last_info.get("server", {}).get("build", {})
+    build_info = getattr(client, "server_last_info", {}).get("server", {}).get("build", {})
 
     def cattr(name):
         return build_info.get(name, "") or getattr(client, f"_remote_build_{name}", "")
@@ -199,9 +224,11 @@ def get_server_builddate(client) -> str:
 
 def get_local_builddate() -> str:
     try:
-        from xpra.build_info import BUILD_DATE, BUILD_TIME
-        return make_datetime(BUILD_DATE, BUILD_TIME)
-    except ImportError:
+        from xpra.build_info import build
+        date = build["date"]
+        time = build["time"]
+        return make_datetime(date, time)
+    except (ImportError, KeyError):
         return ""
 
 
@@ -214,7 +241,7 @@ def get_server_revision_str(client) -> str:
     def cattr(name):
         return getattr(client, name, "")
 
-    build_info = client.server_last_info.get("server", {}).get("build", {})
+    build_info = getattr(client, "server_last_info", {}).get("server", {}).get("build", {})
     if build_info:
         return caps_to_revision(typedict(build_info))
 
@@ -248,7 +275,7 @@ def get_glbuffering_info(opengl_props: dict) -> str:
     return " ".join(info)
 
 
-def set_graph_surface(graph, surface):
+def set_graph_surface(graph, surface) -> None:
     w = surface.get_width()
     h = surface.get_height()
     graph.set_size_request(w, h)
@@ -290,7 +317,7 @@ def save_graph(_ebox, btn, graph) -> None:
             b = BytesIO()
             surface.write_to_png(b)
 
-            def save_file():
+            def save_file() -> None:
                 with open(filename, "wb") as f:
                     f.write(b.getvalue())
 
@@ -312,7 +339,7 @@ class SessionInfo(Gtk.Window):
         self.show_client = show_client
         self.show_server = show_server
         super().__init__()
-        self.last_populate_time = 0
+        self.last_populate_time = 0.0
         self.last_populate_statistics = 0
         self.is_closed = False
         self.set_title(self.get_window_title())
@@ -345,7 +372,7 @@ class SessionInfo(Gtk.Window):
 
         self.set_border_width(15)
 
-        def window_deleted(*_args):
+        def window_deleted(*_args) -> None:
             self.is_closed = True
 
         self.connect('delete_event', window_deleted)
@@ -375,23 +402,23 @@ class SessionInfo(Gtk.Window):
         v = " - ".join(str(x) for x in t)
         return v
 
-    def add_row(self, *widgets):
+    def add_row(self, *widgets) -> None:
         r = int(self.row)
         for i, widget in enumerate(widgets):
             self.grid.attach(widget, i, r, 1, 1)
         self.row.increase()
 
-    def new_row(self, text: str, *widgets):
+    def new_row(self, text: str, *widgets) -> None:
         self.add_row(slabel(text), *widgets)
 
-    def label_row(self, text: str, value=""):
+    def label_row(self, text: str, value="") -> Gtk.Label:
         lbl = slabel(value)
         al = Gtk.Alignment(xalign=0, yalign=0.5, xscale=0.0, yscale=0.0)
         al.add(lbl)
         self.add_row(title_box(text), al)
         return lbl
 
-    def clrow(self, label_str: str, client_label, server_label):
+    def clrow(self, label_str: str, client_label, server_label) -> None:
         labels = []
         if self.show_client:
             labels.append(client_label)
@@ -399,14 +426,14 @@ class SessionInfo(Gtk.Window):
             labels.append(server_label)
         self.add_row(title_box(label_str), *labels)
 
-    def csrow(self, label_str, client_str, server_str):
+    def csrow(self, label_str, client_str, server_str) -> None:
         self.clrow(
             label_str,
             slabel(client_str) if self.show_client else None,
             slabel(server_str) if self.show_server else None,
         )
 
-    def grid_tab(self, icon_filename: str, title: str, populate_cb: Callable):
+    def grid_tab(self, icon_filename: str, title: str, populate_cb: Callable) -> Gtk.VBox:
         self.grid = Gtk.Grid()
         self.row.set(0)
         vbox = self.vbox_tab(icon_filename, title, populate_cb)
@@ -415,12 +442,12 @@ class SessionInfo(Gtk.Window):
         vbox.pack_start(al, expand=True, fill=True, padding=20)
         return vbox
 
-    def vbox_tab(self, icon_filename: str, title: str, populate_cb: Callable):
+    def vbox_tab(self, icon_filename: str, title: str, populate_cb: Callable) -> Gtk.VBox:
         vbox = Gtk.VBox(homogeneous=False, spacing=0)
         self.add_tab(icon_filename, title, populate_cb, contents=vbox)
         return vbox
 
-    def add_tab(self, icon_filename: str, title: str, populate_cb: Callable, contents):
+    def add_tab(self, icon_filename: str, title: str, populate_cb: Callable, contents) -> None:
         icon = get_icon_pixbuf(icon_filename)
 
         def show_tab(*_args):
@@ -432,7 +459,7 @@ class SessionInfo(Gtk.Window):
         self.tab_button_box.add(button)
         self.tabs.append((title, button, contents, populate_cb))
 
-    def show_tab(self, grid):
+    def show_tab(self, grid) -> None:
         button = None
         for _, b, t, p_cb in self.tabs:
             if t == grid:
@@ -454,7 +481,7 @@ class SessionInfo(Gtk.Window):
             alloc = self.get_allocation()
             window.invalidate_rect(alloc, True)
 
-    def set_args(self, *args):
+    def set_args(self, *args) -> None:
         # this is a generic way for keyboard shortcuts or remote commands
         # to pass parameters to us
         log("set_args%s", args)
@@ -471,7 +498,7 @@ class SessionInfo(Gtk.Window):
             log.warn("could not find session info tab named: %s", title)
         log.warn("The options for tab names are: %s)", [x[0] for x in self.tabs])
 
-    def populate_all(self):
+    def populate_all(self) -> None:
         for tab in self.tabs:
             p_cb = tab[3]
             if p_cb:
@@ -499,7 +526,7 @@ class SessionInfo(Gtk.Window):
         self.graph_box.add(button)
         return graph
 
-    def populate_audio_stats(self, *_args):
+    def populate_audio_stats(self, *_args) -> bool:
         # runs every 100ms
         if self.is_closed:
             return False
@@ -517,7 +544,10 @@ class SessionInfo(Gtk.Window):
                 self.audio_out_queue_max.append(qlookup("max"))
         return not self.is_closed
 
-    def populate(self, *_args):
+    def last_info(self) -> dict:
+        return dict(getattr(self.client, "server_last_info", {}))
+
+    def populate(self, *_args) -> bool:
         conn = self.connection
         if self.is_closed or not conn:
             return False
@@ -536,7 +566,7 @@ class SessionInfo(Gtk.Window):
                 if self.client.audio_out_bytecount > 0:
                     self.audio_out_bitcount.append(self.client.audio_out_bytecount * 8)
 
-        if self.show_client and features.windows:
+        if self.show_client and features.window:
             # count pixels in the last second:
             since = monotonic() - 1
             decoded = [0] + [pixels for _, t, pixels in self.client.pixel_counter if t > since]
@@ -546,7 +576,7 @@ class SessionInfo(Gtk.Window):
         # there may be more than one record for each second,
         # so we have to average them to prevent the graph from "jumping":
 
-        def get_ping_latency_records(src, size=25):
+        def get_ping_latency_records(src, size=25) -> tuple:
             recs = {}
             src_list = list(src)
             now = int(monotonic())
@@ -569,14 +599,15 @@ class SessionInfo(Gtk.Window):
 
         self.server_latency = get_ping_latency_records(self.client.server_ping_latency)
         self.client_latency = get_ping_latency_records(self.client.client_ping_latency)
-        if self.client.server_last_info:
+        server_last_info = self.last_info()
+        if server_last_info:
             # populate running averages for graphs:
 
             def getavg(*names):
-                return newdictlook(self.client.server_last_info, list(names) + ["avg"]) or \
-                    newdictlook(self.client.server_last_info, ["client"] + list(names) + ["avg"])
+                return newdictlook(server_last_info, list(names) + ["avg"]) or \
+                    newdictlook(server_last_info, ["client"] + list(names) + ["avg"])
 
-            def addavg(l, *names):
+            def addavg(l, *names) -> None:
                 v = getavg(*names)
                 if v:
                     l.append(v)
@@ -587,7 +618,7 @@ class SessionInfo(Gtk.Window):
             cpl = tuple(1000.0 * x[1] for x in tuple(self.client.client_ping_latency))
             if spl and cpl:
                 self.avg_ping_latency.append(round(sum(spl + cpl) / len(spl + cpl)))
-            if features.windows and self.show_client:
+            if features.window and self.show_client:
                 pc = tuple(self.client.pixel_counter)
                 if pc:
                     tsize = 0
@@ -609,7 +640,7 @@ class SessionInfo(Gtk.Window):
             self.avg_total.append(round(sum(totals)))
         return not self.is_closed
 
-    def add_software_tab(self):
+    def add_software_tab(self) -> None:
         # Package Table:
         self.grid_tab("package.png", "Software", noop)
         if self.show_client and self.show_server:
@@ -619,7 +650,7 @@ class SessionInfo(Gtk.Window):
         self.csrow("Revision", revision_str(), get_server_revision_str(self.client))
         self.csrow("Build date", get_local_builddate(), get_server_builddate(self.client))
 
-        def server_vinfo(lib):
+        def server_vinfo(lib) -> str:
             rlv = getattr(self.client, "_remote_lib_versions", {})
             return make_version_str(rlv.get(lib, ""))
 
@@ -632,12 +663,12 @@ class SessionInfo(Gtk.Window):
         gst_version = props.strtupleget("gst.version")
         self.csrow("GStreamer", make_version_str(gst_version), server_vinfo("sound.gst") or server_vinfo("gst") or "unknown")
 
-        def clientgl(prop="opengl", default_value="n/a"):
+        def clientgl(prop="opengl", default_value="n/a") -> str:
             if not self.show_client:
                 return ""
             return make_version_str(self.client.opengl_props.get(prop, default_value))
 
-        def servergl(prop="opengl", default_value="n/a"):
+        def servergl(prop="opengl", default_value="n/a") -> str:
             if not self.show_server:
                 return ""
             return make_version_str(typedict(self.client.server_opengl or {}).get(prop, default_value))
@@ -646,14 +677,14 @@ class SessionInfo(Gtk.Window):
             key = prop.lower()
             self.csrow(prop, clientgl(key), servergl(key))
 
-    def init_counters(self):
+    def init_counters(self) -> None:
         self.avg_batch_delay = deque(maxlen=N_SAMPLES + 4)
         self.avg_damage_out_latency = deque(maxlen=N_SAMPLES + 4)
         self.avg_ping_latency = deque(maxlen=N_SAMPLES + 4)
         self.avg_decoding_latency = deque(maxlen=N_SAMPLES + 4)
         self.avg_total = deque(maxlen=N_SAMPLES + 4)
 
-    def populate_tab(self, *_args):
+    def populate_tab(self, *_args) -> bool:
         if self.is_closed:
             return False
         # now re-populate the tab we are seeing:
@@ -661,7 +692,7 @@ class SessionInfo(Gtk.Window):
             self.populate_cb = None
         return not self.is_closed
 
-    def show_opengl_state(self):
+    def show_opengl_state(self) -> None:
         if self.client.opengl_enabled:
             glinfo = "%s / %s" % (
                 self.client.opengl_props.get("vendor", ""),
@@ -675,8 +706,8 @@ class SessionInfo(Gtk.Window):
         self.client_opengl_label.set_text(glinfo)
         self.opengl_buffering.set_text(info)
 
-    def show_window_renderers(self):
-        if not features.windows:
+    def show_window_renderers(self) -> None:
+        if not features.window:
             return
         wr = []
         renderers = {}
@@ -686,11 +717,11 @@ class SessionInfo(Gtk.Window):
             wr.append("%s (%i)" % (bclass.__name__.replace("Backing", ""), len(windows)))
         self.window_rendering.set_text("GTK3: %s" % csv(wr))
 
-    def add_features_tab(self):
+    def add_features_tab(self) -> None:
         # Features Table:
         self.grid_tab("features.png", "Features", self.populate_features)
 
-        def image_row(text: str):
+        def image_row(text: str) -> Gtk.Image:
             img = Gtk.Image()
             img.set_margin_start(5)
             al = Gtk.Alignment(xalign=0, yalign=0.5, xscale=0.0, yscale=0.0)
@@ -710,15 +741,15 @@ class SessionInfo(Gtk.Window):
             self.server_mmap_icon = image_row("Memory Mapped Transfers")
         if features.clipboard:
             self.server_clipboard_icon = image_row("Clipboard")
-        if features.notifications:
+        if features.notification:
             self.server_notifications_icon = image_row("Notifications")
-        if features.windows:
+        if features.window:
             self.server_bell_icon = image_row("Bell")
             self.server_cursors_icon = image_row("Cursors")
 
-    def populate_features(self):
+    def populate_features(self) -> bool:
         size_info = ""
-        if features.windows:
+        if features.window:
             if self.client.server_actual_desktop_size:
                 w, h = self.client.server_actual_desktop_size
                 size_info = f"{w}x{h}"
@@ -733,7 +764,7 @@ class SessionInfo(Gtk.Window):
         self.server_randr_label.set_text("%s" % size_info)
         if self.show_client:
             root_w, root_h = self.client.get_root_size()
-            if features.windows and (self.client.xscale != 1 or self.client.yscale != 1):
+            if features.window and (self.client.xscale != 1 or self.client.yscale != 1):
                 sw, sh = self.client.cp(root_w, root_h)
                 display_info = "%ix%i (scaled from %ix%i)" % (sw, sh, root_w, root_h)
             else:
@@ -743,16 +774,18 @@ class SessionInfo(Gtk.Window):
             self.show_window_renderers()
 
         if features.mmap:
-            bool_icon(self.server_mmap_icon, self.client.mmap_enabled)
+            bool_icon(self.server_mmap_icon, bool(self.client.mmap_read_area))
         if features.clipboard:
             bool_icon(self.server_clipboard_icon, self.client.server_clipboard)
-        if features.notifications:
+        if features.notification:
             bool_icon(self.server_notifications_icon, self.client.server_notifications)
-        if features.windows:
+        if features.window:
             bool_icon(self.server_bell_icon, self.client.server_bell)
+        if features.cursor:
             bool_icon(self.server_cursors_icon, self.client.server_cursors)
+        return True
 
-    def add_codecs_tab(self):
+    def add_codecs_tab(self) -> None:
         # Codecs Table:
         self.grid_tab("encoding.png", "Codecs", self.populate_codecs)
         if self.show_client and self.show_server:
@@ -777,7 +810,7 @@ class SessionInfo(Gtk.Window):
         self.server_packet_compressors_label = slabel()
         self.clrow("Packet Compressors", self.client_packet_compressors_label, self.server_packet_compressors_label)
 
-    def populate_codecs(self):
+    def populate_codecs(self) -> bool:
         # clamp the large labels so they will overflow vertically:
         w = self.tab_box.get_preferred_width()[0]
         lw = max(200, int(w // 2.5))
@@ -792,7 +825,7 @@ class SessionInfo(Gtk.Window):
 
         # audio/video codec table:
 
-        def codec_info(enabled, codecs):
+        def codec_info(enabled, codecs) -> str:
             if not enabled:
                 return "n/a"
             return ", ".join(codecs or ())
@@ -812,7 +845,7 @@ class SessionInfo(Gtk.Window):
                 self.client_microphone_codecs_label.set_text(
                     codec_info(c.microphone_allowed, c.microphone_codecs))
 
-        def encliststr(v):
+        def encliststr(v) -> str:
             v = list(v)
             try:
                 v.remove("rgb")
@@ -842,7 +875,7 @@ class SessionInfo(Gtk.Window):
             self.server_packet_compressors_label.set_text(csv(self.client.server_compressors))
         return False
 
-    def add_connection_tab(self):
+    def add_connection_tab(self) -> None:
         self.grid_tab("connect.png", "Connection", self.populate_connection)
 
         def cattr(name) -> str:
@@ -853,8 +886,8 @@ class SessionInfo(Gtk.Window):
         if features.display and self.client.server_display:
             self.label_row("Server Display", prettify_plug_name(self.client.server_display))
         self.label_row("Server Hostname", cattr("_remote_hostname"))
-        if cattr("server_platform"):
-            self.label_row("Server Platform", cattr("server_platform"))
+        if cattr("_remote_platform"):
+            self.label_row("Server Platform", cattr("_remote_platform"))
         self.server_load_label = self.label_row("Server Load")
         self.server_load_label.set_tooltip_text("Average over 1, 5 and 15 minutes")
         self.session_started_label = self.label_row("Session Started")
@@ -870,7 +903,7 @@ class SessionInfo(Gtk.Window):
         self.input_encryption_label = self.label_row("Input Encryption")
         self.output_encryption_label = self.label_row("Output Encryption")
 
-        def add_audio_row(text):
+        def add_audio_row(text) -> tuple[Gtk.Label, Gtk.Label]:
             lbl = slabel()
             al = Gtk.Alignment(xalign=0, yalign=0.5, xscale=0.0, yscale=0.0)
             al.add(lbl)
@@ -881,7 +914,7 @@ class SessionInfo(Gtk.Window):
         self.speaker_label, self.speaker_details = add_audio_row("Speaker")
         self.microphone_label, self.microphone_details = add_audio_row("Microphone")
 
-    def populate_connection(self):
+    def populate_connection(self) -> bool:
         if self.client.server_load:
             self.server_load_label.set_text("  ".join("%.1f" % (x / 1000.0) for x in self.client.server_load))
         if self.client.server_start_time > 0:
@@ -913,14 +946,14 @@ class SessionInfo(Gtk.Window):
 
         if features.audio:
 
-            def get_audio_info(supported, prop):
+            def get_audio_info(supported, prop) -> dict:
                 if not supported:
                     return {"state": "disabled"}
                 if prop is None:
                     return {"state": "inactive"}
                 return prop.get_info()
 
-            def set_audio_info(label, details, supported, prop):
+            def set_audio_info(label, details, supported, prop) -> None:
                 d = typedict(get_audio_info(supported, prop))
                 state = d.strget("state")
                 codec_descr = d.strget("codec") or d.strget("codec_description")
@@ -957,7 +990,7 @@ class SessionInfo(Gtk.Window):
         from xpra.net.crypto import get_crypto_caps
         ccaps = get_crypto_caps()
 
-        def enclabel(label_widget, cipher):
+        def enclabel(label_widget, cipher) -> None:
             if not cipher:
                 info = "None"
             else:
@@ -974,30 +1007,32 @@ class SessionInfo(Gtk.Window):
         return True
 
     def getval(self, prefix, suffix, alt=""):
-        if self.client.server_last_info is None:
+        server_last_info = self.last_info()
+        if not server_last_info:
             return ""
         altv = ""
         if alt:
-            altv = dictlook(self.client.server_last_info, (alt + "." + suffix).encode(), "")
-        return dictlook(self.client.server_last_info, (prefix + "." + suffix).encode(), altv)
+            altv = dictlook(server_last_info, (alt + "." + suffix).encode(), "")
+        return dictlook(server_last_info, (prefix + "." + suffix).encode(), altv)
 
-    def values_from_info(self, prefix, alt=None):
+    def values_from_info(self, prefix, alt=None) -> tuple:
         def getv(suffix):
             return self.getval(prefix, suffix, alt)
 
         return getv("cur"), getv("min"), getv("avg"), getv("90p"), getv("max")
 
-    def all_values_from_info(self, *window_props):
+    def all_values_from_info(self, *window_props) -> tuple[int, int, int, int, int]:
+        server_last_info = self.last_info()
         for window_prop in window_props:
             prop_path = "client.%s" % window_prop
-            v = dictlook(self.client.server_last_info, prop_path)
+            v = dictlook(server_last_info, prop_path)
             if v is not None:
                 v = typedict(v)
                 iget = v.intget
                 return iget("cur"), iget("min"), iget("avg"), iget("90p"), iget("max")
         return 0, 0, 0, 0, 0
 
-    def add_statistics_tab(self):
+    def add_statistics_tab(self) -> None:
         # Details:
         vbox = self.grid_tab("browse.png", "Statistics", self.populate_statistics)
         self.add_row(*(title_box(x) for x in ("", "Latest", "Minimum", "Average", "90 percentile", "Maximum")))
@@ -1015,7 +1050,7 @@ class SessionInfo(Gtk.Window):
             "Client Latency (ms)",
             "The time it takes for the client to respond to pings, as measured by the server",
         )
-        if not features.windows or not self.client.windows_enabled:
+        if not features.window or not self.client.windows_enabled:
             return
         self.batch_labels = maths_labels(
             "Batch Delay (MPixels / ms)",
@@ -1071,7 +1106,7 @@ class SessionInfo(Gtk.Window):
         al.add(self.encoder_info_box)
         vbox.add(al)
 
-    def populate_statistics(self):
+    def populate_statistics(self) -> bool:
         log("populate_statistics()")
         if monotonic() - self.last_populate_statistics < 1.0:
             # don't repopulate more than every second
@@ -1079,36 +1114,13 @@ class SessionInfo(Gtk.Window):
         self.last_populate_statistics = monotonic()
         self.client.send_info_request()
 
-        def setall(labels, values):
-            assert len(labels) == len(values), "%s labels and %s values (%s vs %s)" % (
-                len(labels), len(values), labels, values)
-            for i, l in enumerate(labels):
-                l.set_text(str(values[i]))
-
-        def setlabels(labels, values, rounding: Callable = int):
-            if not values:
-                return
-            avg = sum(values) / len(values)
-            svalues = sorted(values)
-            l = len(svalues)
-            assert l > 0
-            if l < 10:
-                index = l - 1
-            else:
-                index = int(l * 90 / 100)
-            index = max(0, min(l - 1, index))
-            pct = svalues[index]
-            disp = values[-1], min(values), avg, pct, max(values)
-            rounded_values = [rounding(v) for v in disp]
-            setall(labels, rounded_values)
-
         if self.client.server_ping_latency:
             spl = tuple(int(1000 * x[1]) for x in tuple(self.client.server_ping_latency))
             setlabels(self.server_latency_labels, spl)
         if self.client.client_ping_latency:
             cpl = tuple(int(1000 * x[1]) for x in tuple(self.client.client_ping_latency))
             setlabels(self.client_latency_labels, cpl)
-        if features.windows and self.client.windows_enabled:
+        if features.window and self.client.windows_enabled:
             setall(self.batch_labels, self.values_from_info("batch_delay", "batch.delay"))
             setall(self.damage_labels, self.values_from_info("damage_out_latency", "damage.out_latency"))
             setall(self.quality_labels, self.all_values_from_info("quality", "encoding.quality"))
@@ -1163,7 +1175,8 @@ class SessionInfo(Gtk.Window):
             # remove all the current labels:
             for x in self.encoder_info_box.get_children():
                 self.encoder_info_box.remove(x)
-            if self.client.server_last_info:
+            server_last_info = self.last_info()
+            if server_last_info:
                 window_encoder_stats = self.get_window_encoder_stats()
                 # log("window_encoder_stats=%s", window_encoder_stats)
                 for wid, props in window_encoder_stats.items():
@@ -1174,10 +1187,11 @@ class SessionInfo(Gtk.Window):
                     self.encoder_info_box.add(lbl)
         return True
 
-    def get_window_encoder_stats(self):
+    def get_window_encoder_stats(self) -> dict:
         window_encoder_stats = {}
         # new-style server with namespace (easier):
-        window_dict = self.client.server_last_info.get("window")
+        server_last_info = self.last_info()
+        window_dict = server_last_info.get("window")
         if window_dict and isinstance(window_dict, dict):
             for k, v in window_dict.items():
                 with log.trap_error("Error: cannot lookup window dict"):
@@ -1187,7 +1201,7 @@ class SessionInfo(Gtk.Window):
                         window_encoder_stats[wid] = encoder_stats
         return window_encoder_stats
 
-    def add_graphs_tab(self):
+    def add_graphs_tab(self) -> None:
         self.graph_box = Gtk.VBox(homogeneous=False, spacing=10)
         self.add_tab("statistics.png", "Graphs", self.populate_graphs, self.graph_box)
         bandwidth_label = "Bandwidth used"
@@ -1209,7 +1223,7 @@ class SessionInfo(Gtk.Window):
         self.audio_out_queue_max = deque(maxlen=N_SAMPLES * 10 + 4)
         self.audio_out_queue_cur = deque(maxlen=N_SAMPLES * 10 + 4)
 
-    def populate_graphs(self, *_args):
+    def populate_graphs(self, *_args) -> bool:
         # older servers have 'batch' at top level,
         # newer servers store it under client
         self.client.send_info_request("network", "damage", "state", "batch", "client")
@@ -1232,7 +1246,7 @@ class SessionInfo(Gtk.Window):
         # bandwidth graph:
         labels, datasets = [], []
 
-        def unit(scale):
+        def unit(scale) -> str:
             if scale == 1:
                 return ""
             unit, value = to_std_unit(scale)
@@ -1251,7 +1265,7 @@ class SessionInfo(Gtk.Window):
             else:
                 labels += ["recv %sb/s" % unit(net_in_scale)]
                 datasets += [net_in_data]
-        if features.windows and SHOW_PIXEL_STATS and self.client.windows_enabled:
+        if features.window and SHOW_PIXEL_STATS and self.client.windows_enabled:
             pixel_scale, in_pixels = values_to_scaled_values(tuple(
                 self.pixel_in_data)[3:N_SAMPLES + 4], min_scaled_value=100)
             datasets.append(in_pixels)
@@ -1274,7 +1288,7 @@ class SessionInfo(Gtk.Window):
                                               start_x_offset=start_x_offset)
             set_graph_surface(self.bandwidth_graph, surface)
 
-        def norm_lists(items, size=N_SAMPLES):
+        def norm_lists(items, size=N_SAMPLES) -> tuple:
             # ensures we always have exactly 20 values,
             # (and skip if we don't have any)
             values, labels = [], []
@@ -1327,7 +1341,7 @@ class SessionInfo(Gtk.Window):
             set_graph_surface(self.audio_queue_graph, surface)
         return True
 
-    def close(self, *args):
+    def close(self, *args) -> None:
         log("SessionInfo.close(%s) is_closed=%s", args, self.is_closed)
         self.is_closed = True
         super().close()
@@ -1337,34 +1351,34 @@ class SessionInfo(Gtk.Window):
 class SessionInfoClient(InfoTimerClient):
     REFRESH_RATE = envint("XPRA_INFO_REFRESH_RATE", 2)
 
-    def setup_connection(self, conn):
+    def __init__(self, *args):
+        super().__init__(*args)
+        self.client_type = "session-info"
+
+    def setup_connection(self, conn) -> None:
         self.session_name = self.server_session_name = "session-info"
-        self.mmap_enabled = False
         self.windows_enabled = False
         self.send_ping = noop
         self.server_audio_send = self.server_audio_receive = True
         self.server_audio_encoders = self.server_audio_decoders = []
         self.server_ping_latency = self.client_ping_latency = []
         self.server_start_time = 0
-        protocol = super().setup_connection(conn)
+        super().setup_connection(conn)
         self.window = None
-        return protocol
 
-    def run_loop(self):
+    def run_loop(self) -> None:
         Gtk.main()
 
-    def exit_loop(self):
+    def exit_loop(self) -> None:
         Gtk.main_quit()
 
-    def client_type(self):
-        return "session-info"
-
-    def update_screen(self):
+    def update_screen(self) -> None:
         # this is called every time we get the server info back
-        log("update_screen() server_last_info=%s", Ellipsizer(self.server_last_info))
-        if not self.server_last_info:
+        server_last_info = self.last_info()
+        log("update_screen() server_last_info=%s", Ellipsizer(server_last_info))
+        if not server_last_info:
             return
-        td = typedict(self.server_last_info)
+        td = typedict(server_last_info)
 
         def rtdict(*keys):
             d = td
@@ -1376,7 +1390,7 @@ class SessionInfoClient(InfoTimerClient):
         from xpra.client.base.serverinfo import get_remote_lib_versions
         features = rtdict("features")
         self.server_clipboard = features.boolget("clipboard")
-        self.server_notifications = features.boolget("notifications")
+        self.server_notifications = features.boolget("notification")
         display = rtdict("display")
         self.server_opengl = rtdict("display", "opengl")
         self.server_bell = display.boolget("bell")
@@ -1409,7 +1423,7 @@ class SessionInfoClient(InfoTimerClient):
         self.server_display = server.strget("display")
         GLib.idle_add(self.do_update_screen)
 
-    def do_update_screen(self):
+    def do_update_screen(self) -> bool:
         if not self.window:
             self.window = SessionInfo(self, "session-info", self._protocol._conn, show_client=False)
             self.window.show_all()
