@@ -1,7 +1,7 @@
 #!/usr/bin/env python3
 # This file is part of Xpra.
-# Copyright (C) 2011-2024 Antoine Martin <antoine@xpra.org>
-# Copyright (C) 2008, 2009, 2010 Nathaniel Smith <njs@pobox.com>
+# Copyright (C) 2011 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2008 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -18,11 +18,13 @@ from xpra.util.str_fn import strtobytes, memoryview_to_bytes
 from xpra.util.env import envbool
 
 # all the encoders we know about:
-ALL_ENCODERS: Sequence[str] = ("rencodeplus", "none")
+ALL_ENCODERS: Sequence[str] = ("rencodeplus", "rencode", "bencode", "yaml", "none")
+
+VALID_ENCODERS: Sequence[str] = ("rencodeplus", "yaml", "none")
 # the encoders we may have, in the best compatibility order
-TRY_ENCODERS: Sequence[str] = ("rencodeplus", "none")
+TRY_ENCODERS: Sequence[str] = ("rencodeplus", "yaml", "none")
 # order for performance:
-PERFORMANCE_ORDER: Sequence[str] = ("rencodeplus",)
+PERFORMANCE_ORDER: Sequence[str] = ("rencodeplus", "yaml", )
 
 
 @dataclass
@@ -39,7 +41,7 @@ ENCODERS: dict[str, PacketEncoder] = {}
 
 def init_rencodeplus() -> PacketEncoder:
     from xpra.net.rencodeplus import rencodeplus  # type: ignore[attr-defined]
-    rencodeplus_dumps = rencodeplus.dumps  # @UndefinedVariable
+    rencodeplus_dumps = rencodeplus.dumps
 
     def do_rencodeplus(value) -> tuple[SizedBuffer, int]:
         return rencodeplus_dumps(value), FLAGS_RENCODEPLUS
@@ -47,6 +49,23 @@ def init_rencodeplus() -> PacketEncoder:
     return PacketEncoder("rencodeplus", FLAGS_RENCODEPLUS, rencodeplus.__version__, do_rencodeplus, rencodeplus.loads)
 
 
+def init_yaml() -> PacketEncoder:
+    import yaml
+
+    def represent_tuple(dumper, value):
+        return dumper.represent_list(value)
+
+    yaml.add_representer(tuple, represent_tuple)
+
+    def yaml_encode(value) -> tuple[SizedBuffer, int]:
+        return yaml.dump(value).encode("utf-8"), FLAGS_YAML
+
+    def yaml_decode(value) -> Any:
+        return yaml.load(value.decode("utf-8"), Loader=yaml.SafeLoader)
+
+    return PacketEncoder("yaml", FLAGS_YAML, yaml.__version__, yaml_encode, yaml_decode)
+
+
 def b(value) -> bytes:
     if isinstance(value, Iterable) and not isinstance(value, (str, bytes, memoryview, bytearray)):
         return b" ".join(b(item) for item in value)
@@ -67,7 +86,7 @@ def init_none() -> PacketEncoder:
     return PacketEncoder("none", FLAGS_NOHEADER, "0", none_encode, none_decode)
 
 
-def init_encoders(*names) -> None:
+def init_encoders(*names: str) -> None:
     for x in names:
         if x not in TRY_ENCODERS:
             logger = Logger("network", "protocol")
@@ -108,7 +127,7 @@ def get_enabled_encoders(order: Iterable[str] = TRY_ENCODERS) -> Sequence[str]:
 
 
 def get_encoder(e) -> Callable:
-    if e not in ALL_ENCODERS:
+    if e not in VALID_ENCODERS:
         raise ValueError(f"invalid encoder name {e!r}")
     if e not in ENCODERS:
         raise ValueError(f"{e!r} is not available")
@@ -148,12 +167,13 @@ def decode(data, protocol_flags: int):
     raise InvalidPacketEncodingException(f"{ptype!r} decoder is not available")
 
 
-def main():  # pragma: no cover
+def main() -> int:  # pragma: no cover
     from xpra.util.str_fn import print_nested_dict
     from xpra.platform import program_context
     with program_context("Packet Encoding", "Packet Encoding Info"):
         init_all()
         print_nested_dict(get_packet_encoding_caps())
+    return 0
 
 
 if __name__ == "__main__":  # pragma: no cover
